import json
import logging
import re
from multiprocessing import Process, Queue

import click
import jsonschema

from inventory_provider import constants
from inventory_provider import snmp
from inventory_provider import juniper




def _load_oids(config_file):
    """
    :param config_file: file-like object
    :return:
    """
    result = {}
    for line in config_file:
        m = re.match(r'^([^=]+)=(.*)\s*$', line)
        if m:
            result[m.group(1)] = m.group(2)
    return result


def _load_routers(config_file):
    """
    :param config_file: file-like object
    :return:
    """
    for line in config_file:
        m = re.match(r'^([a-z\d]+\.[a-z\d]{3,4}\.[a-z\d]{2}\.(geant|eumedconnect)\d*\.net)\s*=([^,]+)\s*,(.*)\s*$', line)
        if not m:
            logging.warning("malformed config file line: '%s'" % line.strip())
            continue
        yield {
            "hostname": m.group(1),
            "community": m.group(3),
            "address": m.group(4)
        }


def _validate_config(ctx, param, value):
    """
    loads, validates and returns configuration parameters

    :param ctx:
    :param param:
    :param value:
    :return:
    """
    config = json.loads(value.read())
    jsonschema.validate(config, constants.CONFIG_SCHEMA)
    with open(config["oid_list.conf"]) as f:
        config["oids"] = _load_oids(f)
    with open(config["routers_community.conf"]) as f:
        config["routers"] = list(_load_routers(f))
    return config








def get_router_interfaces_q(router, config, q):
    threading_logger = logging.getLogger(constants.THREADING_LOGGER_NAME)
    threading_logger.debug("[ENTER>>] get_router_interfaces_q: %r" % router)
    q.put(list(snmp.get_router_interfaces(router, config)))
    threading_logger.debug("[<<EXIT]  get_router_interfaces_q: %r" % router)


def exec_router_commands_json_q(router, ssh_params, commands, q):
    threading_logger = logging.getLogger(constants.THREADING_LOGGER_NAME)
    threading_logger.debug("[ENTER>>] exec_router_commands_q: %r" % router)
    q.put(list(juniper.exec_router_commands_json(router, ssh_params, commands)))
    threading_logger.debug("[<<EXIT] exec_router_commands_q: %r" % router)


def get_router_details(router, params, q):

    threading_logger = logging.getLogger(constants.THREADING_LOGGER_NAME)

    threading_logger.debug("[ENTER>>]get_router_details: %r" % router)

    commands = [
        'show configuration routing-instances IAS protocols bgp',
        # 'show configuration routing-instances IAS protocols bgp | display set | match neighbor | match description | match "GEANT-IX | GEANT-IX-"',
        # 'show configuration routing-instances IAS protocols bgp | display set | match neighbor | match description | match "GEANT-IXv6 | GEANT-IXv6-"',
        'show configuration logical-systems VRR protocols bgp',
        # 'show configuration logical-systems VRR protocols bgp group VPN-RR-INTERNAL | match "neigh|desc"',
        # 'show configuration logical-systems VRR protocols bgp group VPN-RR | match "neigh|desc"'
        'show interfaces descriptions'
    ]

    snmpifc_proc_queue = Queue()
    snmpifc_proc = Process(target=get_router_interfaces_q, args=(router,snmpifc_proc_queue))
    snmpifc_proc.start()

    commands_proc_queue = Queue()
    commands_proc = Process(target=exec_router_commands_json_q, args=(router, params["ssh"], commands, commands_proc_queue))
    commands_proc.start()

    threading_logger.debug("waiting for commands result: %r" % router)
    command_output = commands_proc_queue.get()
    assert len(command_output) == len(commands)
    result = dict(zip(["bgp", "vrr", "interfaces"], command_output))
    commands_proc.join()
    threading_logger.debug("... got commands result & joined: %r" % router)

    threading_logger.debug("waiting for snmp ifc results: %r" % router)
    result["snmp-interfaces"] = snmpifc_proc_queue.get()
    snmpifc_proc.join()
    threading_logger.debug("... got snmp ifc result & joined: %r" % router)

    q.put(result)

    threading_logger.debug("[<<EXIT]get_router_details: %r" % router)


def load_network_details(config):

    threading_logger = logging.getLogger(constants.THREADING_LOGGER_NAME)

    processes = []
    for r in config["routers"]:
        q = Queue()
        p = Process(target=get_router_details, args=(r, config, q))
        p.start()
        processes.append({"router": r, "process": p, "queue": q})

    result = {}
    for p in processes:
        threading_logger.debug("waiting for get_router_details result: %r" % p["router"])
        result[p["router"]["hostname"]] = p["queue"].get()
        p["process"].join()
        threading_logger.debug("got result and joined get_router_details proc: %r" % p["router"])

    return result


@click.command()
@click.option(
    "--config",
#    required=True,
    type=click.File(),
    help="Configuration filename",

    default=open("config.json"),
    callback=_validate_config)
def cli(config):
    network_details = load_network_details(config)
    filename = "/tmp/router-info.json"
    logging.debug("writing output to: " + filename)
    with open(filename, "w") as f:
        f.write(json.dumps(network_details))


if __name__ == "__main__":
    logging.basicConfig(level=logging.WARNING)
    logging.getLogger(constants.SNMP_LOGGER_NAME).setLevel(logging.DEBUG)
    logging.getLogger(constants.THREADING_LOGGER_NAME).setLevel(logging.INFO)
    logging.getLogger(constants.JUNIPER_LOGGER_NAME).setLevel(logging.DEBUG)
    logging.getLogger(constants.DATABASE_LOGGER_NAME).setLevel(logging.DEBUG)
    cli()

